SUBROUTINE &
    sub_egminterp(k,z_node,c_node,val_node,z_exo,c_out,val_out)
! Interpolates policy and value fns on ENDOGENOUS cash-at-hand grid
!    to obtain policy and value fns on EXOGENOUS grid
! Inputs: 
! 	k (size of vector of interpolating nodes)
!	z_node (vector of ascissas of interpolating nodes)
!       c_node, val_node (ordinated of interpolating nodes for
!               consumption and value fns)
!       z_exo (interpolating points) 
! Outputs:
!	c_out, val_out (consumption and value fn on the exogenous  grid) 


  USE mod_types
  USE mod_common_var  
  USE mod_felicity
  USE mod_int_node

  IMPLICIT NONE
  
  INTEGER,  INTENT(in) :: k
  REAL(dp), INTENT(in), DIMENSION(k) :: c_node, val_node, z_node
  REAL(dp), INTENT(in), DIMENSION(len_a_grid) :: z_exo
  REAL(dp), INTENT(out), DIMENSION(len_a_grid)  :: c_out, val_out

  INTEGER, DIMENSION(len_a_grid)  :: node
  REAL(dp) :: fel1



  ! 1. Compute RIGHT interpolating nodes
  !    The first node identifies borrowing-const. individual
  !----------------------------------------------------------
  CALL sub_integm_node(z_node,z_exo,node)
  node = node - 1
  fel1 = fel(c_node(1))

  
  ! 2. Interpolate policy and value functions 
  !----------------------------------------------------------
  WHERE (node==0)
    ! Borrowing constrained individual 
    WHERE (z_exo.LE.a_grid(1))
      ! If consumption is non-positive assign arbitrarily small consumption
      ! and values
      c_out = 1.d-7
      val_out = -1.d7
    ELSEWHERE
      ! Borrowing constrained agents
      c_out = z_exo - a_grid(1)
      val_out = val_node(1) - fel1 + MIN(fel(c_out),fel1)
    END WHERE
    
  ELSEWHERE 
    c_out = c_node(node)+ &
        ( c_node(node+1)-c_node(node) )/( z_node(node+1)-z_node(node) ) *&
              ( z_exo-z_node(node) )
    val_out = val_node(node)+ &
        ( val_node(node+1)-val_node(node) )/( z_node(node+1)-z_node(node) ) *&
        ( z_exo-z_node(node) )

  END WHERE
END SUBROUTINE sub_egminterp


