MODULE mod_common_var
  ! Assigns values to economic and numerical parameters
  
  USE mod_types
  IMPLICIT NONE

!*********************************************************************
!************* Global variables used by the algorithm. *************** 
!*********************************************************************
! The next six declarations are grouped here just for ease of comparison
! with mod_common_var.f90 for the EGM algorithm

  INTEGER, PARAMETER 	:: len_a_grid = 200   ! Asset grid size
  REAL (dp), DIMENSION(len_a_grid) :: a_grid ! Asset grid


! Global variables used in the sub_routines passed to the 
! non-linear solver

  REAL (dp) :: b_global, fut_v1_global 

! Global flag selecting felicity fn parameterization
  INTEGER :: fel_flag

! Used only if felicity depends on variable h_prime in addition to c
  REAL (dp) :: h_prime

! Global flag = 0 if discretized VFI, =1 if lin. interp. VFI
  INTEGER, PARAMETER :: VFI_type  = 1

!*********************************************************************

!*********************************************************************
! Global variables used ONLY by the application of the code in the paper
!*********************************************************************

  !----------------------------------------------------------------
  ! 1. Economic parameters 
  !----------------------------------------------------------------

  REAL (dp), PARAMETER :: theta		= .77d0! Non-durables weight
  REAL (dp), PARAMETER :: tau  		= 0.d0 !.2435d0	! Intratemporal elast.
  REAL (dp), PARAMETER :: kappa  	= .075d0! Flow of house servic.
  REAL (dp), PARAMETER :: bita		= 0.93d0! Discount factor
  REAL (dp), PARAMETER :: alpha		= 1.d0! CRRA coefficient
 

  ! AR(1) parameters for persistent income shock yp
  ! Unconditional mean mu0 normalized to 1
  REAL (dp), PARAMETER :: rho_yp 	= .977d0
  REAL (dp), PARAMETER :: mu0_yp 	= 0.d0
  REAL (dp), PARAMETER :: sigma_yp 	= .024d0

  ! AR(1) parameters for temporary income shock yt
  REAL (dp), PARAMETER :: rho_yt	= .0d0
  REAL (dp), PARAMETER :: mu0_yt 	= .0d0
  REAL (dp), PARAMETER :: sigma_yt 	= .063d0

  REAL (dp), PARAMETER :: min_a	= 0.d0	! Lower bound on assets
  REAL (dp), PARAMETER :: max_a	= 25.d0 ! Upper bound on assets
  REAL (dp), PARAMETER :: min_h	= 1.d-2	! Lower bound on housing size
  REAL (dp), PARAMETER :: max_h = 10.0  !Upper bound on housing
  REAL (dp), PARAMETER :: r = 6.d-2     ! Interest rate
  REAL (dp), PARAMETER :: downp	= .2d0	! Downpayment ratio
  REAL (dp), PARAMETER :: gamma	= .06d0	! Percentage transaction cost


  !----------------------------------------------------------------
  ! 2. Numerical parameters
  !----------------------------------------------------------------

  INTEGER, PARAMETER :: len_yp_grid = 7	  ! Persistent shock grid size
  REAL(dp), PARAMETER :: cover_yp = 3.d0  ! Cover 3 SD each side

  INTEGER, PARAMETER :: len_yt_grid = 7   ! Persistent shock grid size
  REAL(dp), PARAMETER :: cover_yt = 3.d0  ! Cover 3 SD each side

  INTEGER, PARAMETER :: len_h_grid = 7    ! Number of housing choices

  REAL (dp), PARAMETER :: toler   = 1d-5  ! Numerical tolerance
  INTEGER, PARAMETER :: Tsimul    = 50000 ! Number of simulations for Euler errors

  ! Income grid
  INTEGER, PARAMETER :: len_y_grid = len_yp_grid*len_yt_grid

  ! 3. Define global variables for the grids created in main.f90
  !----------------------------------------
  !
  REAL (dp), DIMENSION(len_y_grid)		:: y_grid
  REAL (dp), DIMENSION(len_h_grid)		:: h_grid
  REAL (dp), DIMENSION(len_y_grid,len_y_grid)	:: pi_y,picum_y
  

END MODULE mod_common_var
