MODULE mod_felicity
! Contain the felicity function (fel) and its first (dfel) and second
! derivative (d2fel).  
! The three of them need to be entered by the USER.  
! Each function has a scalar (_sc) and an array (_ar) version. 
! They both need to be entered.
  USE mod_types
  USE mod_common_var  

  IMPLICIT NONE

  INTERFACE fel
     MODULE PROCEDURE fel_sc, fel_ar
  END INTERFACE

  INTERFACE dfel
     MODULE PROCEDURE dfel_sc, dfel_ar
  END INTERFACE

  INTERFACE d2fel
     MODULE PROCEDURE d2fel_sc, d2fel_ar
  END INTERFACE

CONTAINS

  !******************************************************
  !               Felicity function                     !
  !******************************************************
  FUNCTION fel_sc(con)
    !Returns felicity for a consumption SCALAR 

    REAL(dp), INTENT(IN)        :: con
    REAL(dp)                    :: fel_sc, agg

    SELECT CASE (fel_flag)
      CASE(1)
        agg = (theta*con**tau+(1.d0-theta)*(kappa*h_prime)**tau)**(1.d0/tau)
        fel_sc = (agg**(1.d0-alpha) - 1.d0)/(1.d0-alpha)
      CASE(2)
        fel_sc = LOG(theta*con**tau+(1.d0-theta)*(kappa*h_prime)**tau)/tau
      CASE(3)
        fel_sc = theta*LOG(con)+(1.d0-theta)*LOG(kappa*h_prime)
      CASE(4)
        fel_sc = ( (con**theta*(kappa*h_prime)**(1.d0-theta))**(1.d0-alpha)&
            - 1.d0 )/(1.d0 - alpha)
    END SELECT
    
  END FUNCTION fel_sc


  FUNCTION fel_ar(con)
    !Returns felicity ARRAY for a consumption ARRAY

    REAL(dp), INTENT(IN), DIMENSION(:)	:: con
    REAL(dp), DIMENSION(SIZE(con))	:: fel_ar, agg

    SELECT CASE (fel_flag)
      CASE(1)
        agg = (theta*con**tau+(1.d0-theta)*(kappa*h_prime)**tau)**(1.d0/tau)
        fel_ar = (agg**(1.d0-alpha) - 1.d0)/(1.d0-alpha)
      CASE(2)
        fel_ar = LOG(theta*con**tau+(1.d0-theta)*(kappa*h_prime)**tau)/tau
      CASE(3)
        fel_ar = theta*LOG(con)+(1.d0-theta)*LOG(kappa*h_prime)
      CASE(4)
        fel_ar = ( (con**theta*(kappa*h_prime)**(1.d0-theta))**(1.d0-alpha)&
            - 1.d0 )/(1.d0 - alpha)

    END SELECT

  END FUNCTION fel_ar

  !******************************************************
  !            Marginal felicity function               !
  !******************************************************
  FUNCTION dfel_sc(con)
    !Returns marginal felicity for a consumption SCALAR

    REAL(dp), INTENT(IN)        :: con
    REAL(dp)                    :: dfel_sc, agg
    
    IF (fel_flag.NE.4) THEN
      agg = (theta*con**tau+(1.d0-theta)*(kappa*h_prime)**tau)**(1.d0/tau)
      dfel_sc = theta*con**(tau-1.d0)*(agg)**(1.d0-alpha-tau)
    ELSE
      dfel_sc = theta*con**(theta*(1.d0-alpha)-1.d0)*&
          (kappa*h_prime)**((1.d0-theta)*(1.d0-alpha))
    END IF
    
  END FUNCTION dfel_sc

  FUNCTION dfel_ar(con)
    !Returns marginal felicity for a consumption ARRAY

    REAL(dp), INTENT(IN), DIMENSION(:)	:: con
    REAL(dp), DIMENSION(SIZE(con))	:: dfel_ar, agg

    IF (fel_flag.NE.4) THEN
      agg = (theta*con**tau+(1.d0-theta)*(kappa*h_prime)**tau)**(1.d0/tau)
      dfel_ar = theta*con**(tau-1.d0)*(agg)**(1.d0-alpha-tau)
    ELSE
      dfel_ar = theta*con**(theta*(1.d0-alpha)-1.d0)*&
          (kappa*h_prime)**((1.d0-theta)*(1.d0-alpha))
    END IF

  END FUNCTION dfel_ar

  !******************************************************
  !      Second derivative of felicity function         !
  !******************************************************
  FUNCTION d2fel_sc(con)
    !Returns second der. of felicity for a consumption scalar

    REAL(dp), INTENT(IN)        :: con
    REAL(dp)                    :: d2fel_sc, agg

    IF (fel_flag.NE.4) THEN
      agg = (theta*con**tau+(1.d0-theta)*(kappa*h_prime)**tau)**(1.d0/tau)
      d2fel_sc = (tau-1.d0)*theta*con**(tau-2.d0)*(agg)**(1.d0-alpha-tau) +&
          (1.d0-alpha-tau)*(theta*con**(tau-1.d0))**2.d0*&
          agg**(1.d0-alpha-2.d0*tau)
    ELSE
      d2fel_sc = theta*(theta*(1.d0-alpha)-1.d0)*con**(theta*(1.d0-alpha)-2.d0)*&
        (kappa*h_prime)**((1.d0-theta)*(1.d0-alpha))
    END IF
    

  END FUNCTION d2fel_sc

  FUNCTION d2fel_ar(con)
    !Returns second der. of felicity for a consumption ARRAY

    REAL(dp), INTENT(IN), DIMENSION(:)	:: con
    REAL(dp), DIMENSION(SIZE(con))	:: d2fel_ar, agg

    IF (fel_flag.NE.4) THEN
      agg = (theta*con**tau+(1.d0-theta)*(kappa*h_prime)**tau)**(1.d0/tau)
      d2fel_ar = (tau-1.d0)*theta*con**(tau-2.d0)*(agg)**(1.d0-alpha-tau) +&
          (1.d0-alpha-tau)*(theta*con**(tau-1.d0))**2.d0*&
          agg**(1.d0-alpha-2.d0*tau)
    ELSE
      d2fel_ar = theta*(theta*(1.d0-alpha)-1.d0)*con**(theta*(1.d0-alpha)-2.d0)*&
        (kappa*h_prime)**((1.d0-theta)*(1.d0-alpha))
    END IF

  END FUNCTION d2fel_ar

END MODULE mod_felicity
